@@ -3,9 +3,11 @@
 use super::{ChanCmd, RawCellBody, CELL_DATA_LEN};
 use std::convert::TryInto;
 use std::net::{IpAddr, Ipv4Addr};
+use tor_basic_utils::skip_fmt;
 use tor_bytes::{self, Error, Readable, Reader, Result, Writer};
 
 use caret::caret_int;
+use educe::Educe;
 
 /// Trait for the 'bodies' of channel messages.
 pub trait Body: Readable {
@@ -159,7 +161,7 @@ impl ChanMsg {
 /// channel, or as a "keep-alive".
 ///
 /// The correct response to a padding cell is to drop it and do nothing.
-#[derive(Clone, Debug)]
+#[derive(Clone, Debug, Default)]
 #[non_exhaustive]
 pub struct Padding {}
 impl Padding {
@@ -168,11 +170,6 @@ impl Padding {
         Padding {}
     }
 }
-impl Default for Padding {
-    fn default() -> Self {
-        Padding::new()
-    }
-}
 impl Body for Padding {
     fn into_message(self) -> ChanMsg {
         ChanMsg::Padding(self)
@@ -431,7 +428,8 @@ impl Readable for Created2 {
 ///
 /// A different protocol is defined over the relay cells; it is implemented
 /// in the [crate::relaycell] module.
-#[derive(Clone)]
+#[derive(Clone, Educe)]
+#[educe(Debug)]
 pub struct Relay {
     /// The contents of the relay cell as encoded for transfer.
     ///
@@ -440,6 +438,7 @@ pub struct Relay {
     /// places where we _don't_ Box things that we should, and more copies than
     /// necessary happen. We should refactor our data handling until we're mostly
     /// moving around pointers rather than copying data;  see ticket #7.
+    #[educe(Debug(method = "skip_fmt"))]
     body: Box<RawCellBody>,
 }
 impl Relay {
@@ -472,11 +471,6 @@ impl Relay {
         ChanMsg::RelayEarly(self)
     }
 }
-impl std::fmt::Debug for Relay {
-    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
-        f.debug_struct("Relay").finish()
-    }
-}
 impl Body for Relay {
     fn into_message(self) -> ChanMsg {
         ChanMsg::Relay(self)
@@ -593,8 +587,11 @@ impl DestroyReason {
 /// channel and sending data.
 #[derive(Clone, Debug)]
 pub struct Netinfo {
-    /// Time when this cell was sent, or 0 if this cell is sent by
-    /// a client.
+    /// Time when this cell was sent, or 0 if this cell is sent by a client.
+    ///
+    /// TODO-SPEC(nickm): Y2038 issue here.  Better add a new handshake version
+    /// to solve it.  See
+    /// [torspec#80](https://gitlab.torproject.org/tpo/core/torspec/-/issues/80).
     timestamp: u32,
     /// Observed address for party that did not send the netinfo cell.
     their_addr: Option<IpAddr>,
@@ -658,6 +655,15 @@ impl Netinfo {
             my_addr,
         }
     }
+    /// Return the time reported in this NETINFO cell.
+    pub fn timestamp(&self) -> Option<std::time::SystemTime> {
+        use std::time::{Duration, SystemTime};
+        if self.timestamp == 0 {
+            None
+        } else {
+            Some(SystemTime::UNIX_EPOCH + Duration::from_secs(self.timestamp.into()))
+        }
+    }
 }
 impl Body for Netinfo {
     fn into_message(self) -> ChanMsg {
